@@ -83,6 +83,55 @@ class DateTimeEncoder(json.JSONEncoder):
             return super(DateTimeEncoder, self).default(obj)
 
 
+# Tools for converting 'snake_case' to 'lowerCamelCase' 'UpperCamelCase' and 'Space Camel Case'. Adapted from: https://stackoverflow.com/a/19053800
+def snake_to_camel(snake_str, upper=False, space=False):
+    components = snake_str.split('_')
+    if upper:
+        join_char = ''
+        if space:
+            join_char = ' '
+        # Capitalize the first letter of each component, using title(), then join them together.
+        return join_char.join(x.title() for x in components)
+    else:
+        # Capitalize the first letter of each component except the first one, using title(), then join them together.
+        return components[0] + ''.join(x.title() for x in components[1:])
+
+def camel_to_snake(camelStr):
+      mid_str = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', camelStr)
+      return re.sub('([a-z0-9])([A-Z])', r'\1_\2', mid_str).lower()
+
+# This rewrites the key in-place. Make a copy.deepcopy() before calling this!
+def json_snake_to_camel(json_obj, upper=False, space=False, debug=False):
+    mro = inspect.getmro(json_obj.__class__)
+    if dict in mro:
+        # Since the json_obj is changing in place, grab the keys to iterate up front so the iterator doesn't change during the loop
+        for snake_key in list(json_obj.keys()):
+            value = json_obj.get(snake_key)
+            if debug:
+                logw(snake_key, "SNAKE KEY")
+                logw(value, "VALUE")
+            camel_key = snake_to_camel(snake_key, upper=upper, space=space)
+            if debug:
+                logw(camel_key, "CAMEL KEY")
+            value_mro = inspect.getmro(value.__class__)
+            if dict in value_mro or list in value_mro:
+                if debug:
+                    logw(value, "DICT VALUE BEFORE")
+                json_snake_to_camel(value, upper=upper, space=space, debug=debug)
+                if debug:
+                    logw(value, "DICT VALUE AFTER")
+            json_obj[camel_key] = value
+            if camel_key != snake_key:
+                del json_obj[snake_key]
+    elif list in mro:
+        for item in json_obj:
+            if debug:
+                logw(item, "LIST ITEM BEFORE")
+            json_snake_to_camel(item, upper=upper, space=space, debug=debug)
+            if debug:
+                logw(item, "LIST ITEM AFTER")
+
+
 def ts_to_dt(timestamp):
     if timestamp is None: return None
     return datetime.datetime.utcfromtimestamp(timestamp)
